@import "https://cdn.jsdelivr.net/npm/bulma@0.9.3/css/bulma.min.css";
@import "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.2/css/all.min.css";

/*
<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bulma@0.9.3/css/bulma.min.css">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.2/css/all.min.css"
        integrity="sha512-HK5fgLBL+xu6dm/Ii3z4xhlSUyZgTT9tuc/hSrtw6uzJOvgRr2a9jyxxT1ely+B+xFAmJKVSTbpM/CuL7qxO8w=="
        crossorigin="anonymous" />*/

:root{ 
        --def-transition_speed: all 0.1s;  
        --def-font-family: 'Helvetica Neue', sans-serif;
        --def-letter-spacing: 1px;
        --def-font-weight: 400;


        --light_text_color_cat1_subdued: #b8aeae;

        --light_text_color_cat1: #e2e7c9;
        --light_text_color_cat1_background: #0f0f0f;

        /*--background_cat1_color:#1c232f;
        --background_cat2_color: grey;
        --background_cat3_color: blue;*/

        --background_cat1_color: #415fb3;
        --background_cat2_color: #6b41b3;
        --background_cat3_color: #41b395;
     } 

body {
    font-family: var(--def-font-family);
    letter-spacing: var(--def-letter-spacing);
    font-weight: var(--def-font-weight);
}

#main_area{ 
    transition:  var(--def-transition_speed);  
}

.divider{ 
    align-items: center; 
    height: 1px;
    /*padding: 1em;*/
    /*margin: 1em 1em;*/
    background: #cfc6c6; 
}

 

/*

.xxsc_bor_section_height_small{
    padding: 25px 20px 25px 20px;  
}

.xxsc_has_border_thin{
    padding: 10px;
    border-style: solid  ;
    border-width: 1px  ;
    border-color: rgb(206, 206, 206)   ;
}


.xxsc_card_equal_height{
height: 100%;
display:flex;
flex-direction: column;
}

.xxsc_circle-icon {
    background: #a5eeea;
    padding:20px; 
    border-radius: 50%;
} 

.xxsc_card-footer {
    margin-top: auto;
}
*/ 
.screen_height{
    height: 100%;
    min-height: 100vh;
}
