
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Document</title>


    <!-- <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bulma@0.9.3/css/bulma.min.css">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.2/css/all.min.css"
        integrity="sha512-HK5fgLBL+xu6dm/Ii3z4xhlSUyZgTT9tuc/hSrtw6uzJOvgRr2a9jyxxT1ely+B+xFAmJKVSTbpM/CuL7qxO8w=="
        crossorigin="anonymous" /> -->

    <!-- <link rel="stylesheet" href="/commonui/st/_def/static/css/main_styles.css" crossorigin="anonymous" />  -->
    <link rel="stylesheet" href="/webui/static/zcss/main.css" crossorigin="anonymous" /> 
    <!-- <link rel="stylesheet" href="/commonui/static/_def/css/bulma_common.css" crossorigin="anonymous" />  -->
    <!-- <link rel="stylesheet" href="/commonui/static/_def/css/debug.css" crossorigin="anonymous" />  -->
    
    {% block head_inserts %}
    {% endblock %}

</head>


<body>
    
    <!-- hello 1 world -->

<div class="is-flex screen_height">
    

{% block nav_content %}
{% endblock %}

{% block main_body %}
{% endblock %}
</div>


{% block page_script %}
{% endblock %}

{% block nav_script %}
{% endblock %}




</body>
</html>





