* *:hover { border:2px solid #89A81E }                   
* * *:hover { border:2px solid #F34607 }                  
* * * *:hover { border:2px solid #5984C3 }                
* * * * *:hover { border:2px solid #CD1821 }             
* * * * * *:hover { border:2px dotted #89A81E }           
* * * * * * *:hover { border:2px dotted #F34607 }        
* * * * * * * *:hover { border:2px dotted #5984C3 }       
* * * * * * * * *:hover { border:2px dotted #CD1821 }     